<?php

declare(strict_types=1);

namespace mirzaev\calculator\models;

use exception;
use pdo;

/**
 * Модель баллонов
 *
 * @package mirzaev\calculator\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 *
 * @todo
 * 1. Если длина реза баллона зависит от типа металла (учитывается) то перенести это в класс металлов
 */
final class baloons_model extends core
{
    /**
     * Используемый газ
     */
    public string $gas;

    /**
     * Количество баллонов
     */
    public float $amount = 0;

    /**
     * Цена всех баллонов
     */
    public float $cost;

    /**
     * Вычисление используемого газа
     *
     * @param float $length Толщина металла
     * @param array &$errors Журнал ошибок
     *
     * @return string|null Название газа
     */
    public function gas(float $length, array &$errors = []): ?string
    {
        try {
            return $this->gas = match (true) {
                $length >= 4 => 'oxygen',
                default => 'air'
            };
        } catch (exception $e) {
            // Запись в журнал ошибок
            $errors[] = [
                'text' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'stack' => $e->getTrace()
            ];
        }

        return null;
    }

    /**
     * Вычисление количества используемых баллонов
     *
     * @param string $metal Тип металла
     * @param float $cutting Длина реза
     * @param float $length Толщина листа
     * @param string|null $gas Используемый газ
     * @param array &$errors Журнал ошибок
     *
     * @return int|null Количество баллонов
     *
     * @todo
     * 1. Добавить к баллонам уточнение чтобы считало не по листам а по объёму а лучше по длине реза
     * 2. Определение длины реза по типу металла
     */
    public function amount(string $metal, float $cutting, float $length, ?string $gas = null, array &$errors = []): ?float
    {
        try {
            // Инициализация входных параметров
            $gas ?? $gas = &$this->gas;

            // Инициализация запроса
            $request = static::$db->prepare("SELECT `length` FROM `baloons` WHERE `gas` = :gas LIMIT 30");

            // Отправка запроса
            $request->execute([
                ':gas' => $gas
            ]);

            // Генерация ответа
            $response = $request->fetch(pdo::FETCH_ASSOC);

            // Проверка на полученные значения
            if (!is_array($response)) return null;

            // Вычисление длины реза на которое хватит баллона
            $flow = $response['length'] / $length;

            // Вычисление количества баллонов (округление к большему)
            return $this->amount = $cutting / $flow;
        } catch (exception $e) {
            // Запись в журнал ошибок
            $errors[] = [
                'text' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'stack' => $e->getTrace()
            ];
        }

        return null;
    }



    /**
     * Вычисление стоимости баллонов
     *
     * @param string $metal Тип металла
     * @param int|null $gas Используемый газ
     * @param array &$errors Журнал ошибок
     *
     * @return float|null Стоимость баллонов
     */
    public function cost(string $metal, ?int $amount = null, ?string $gas = null, array &$errors = []): ?float
    {
        try {
            // Инициализация входных параметров
            $amount ?? $amount = &$this->amount;
            $gas ?? $gas = &$this->gas;

            // Инициализация запроса
            $request = static::$db->prepare("SELECT `cost` FROM `baloons` WHERE `gas` = :gas LIMIT 30");

            // Отправка запроса
            $request->execute([
                ':gas' => $gas
            ]);

            // Генерация ответа
            $response = $request->fetch(pdo::FETCH_ASSOC);

            // Проверка на полученные значения
            if (!is_array($response)) return null;

            // Инициализация стоимости всех баллонов
            return $this->cost = $response['cost'] * $amount;
        } catch (exception $e) {
            // Запись в журнал ошибок
            $errors[] = [
                'text' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'stack' => $e->getTrace()
            ];
        }

        return null;
    }
}
